@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -144,26 +144,39 @@ extern pthread_mutex_t *nd_printf_mutex;
 
 void nd_printf(const char *format, ...)
 {
-    pthread_mutex_lock(nd_printf_mutex);
-
     va_list ap;
     va_start(ap, format);
-    vsyslog(LOG_DAEMON | LOG_INFO, format, ap);
+    nd_printf(format, ap);
     va_end(ap);
+}
+
+void nd_printf(const char *format, va_list ap)
+{
+    pthread_mutex_lock(nd_printf_mutex);
+
+    vsyslog(LOG_DAEMON | LOG_INFO, format, ap);
 
     pthread_mutex_unlock(nd_printf_mutex);
 }
 
-void nd_debug_printf(const char *format, ...)
+void nd_dprintf(const char *format, ...)
+{
+    if (ND_DEBUG) {
+
+        va_list ap;
+        va_start(ap, format);
+        nd_dprintf(format, ap);
+        va_end(ap);
+    }
+}
+
+void nd_dprintf(const char *format, va_list ap)
 {
     if (ND_DEBUG) {
 
         pthread_mutex_lock(nd_printf_mutex);
 
-        va_list ap;
-        va_start(ap, format);
         vfprintf(stderr, format, ap);
-        va_end(ap);
 
         pthread_mutex_unlock(nd_printf_mutex);
     }
@@ -225,14 +238,14 @@ void nd_print_address(const struct sockaddr_storage *addr)
             _addr, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
         break;
     default:
-        nd_debug_printf("(unsupported AF:%x)", addr->ss_family);
+        nd_dprintf("(unsupported AF:%x)", addr->ss_family);
         return;
     }
 
     if (rc == 0)
-        nd_debug_printf("%s", _addr);
+        nd_dprintf("%s", _addr);
     else
-        nd_debug_printf("???");
+        nd_dprintf("???");
 }
 
 void nd_print_binary(uint32_t byte)
@@ -244,7 +257,7 @@ void nd_print_binary(uint32_t byte)
     for (i = 0x80000000; i > 0; i >>= 1)
         strcat(b, ((byte & i) == i) ? "1" : "0");
 
-    nd_debug_printf("%s", b);
+    nd_dprintf("%s", b);
 }
 
 void nd_print_number(ostringstream &os, uint64_t value, bool units_binary)
@@ -299,6 +312,20 @@ void nd_print_number(ostringstream &os, uint64_t value, bool units_binary)
     }
 }
 
+static inline void nd_ltrim(string &s)
+{
+    s.erase(s.begin(), find_if(s.begin(), s.end(), [](unsigned char ch) {
+        return !isspace(ch);
+    }));
+}
+
+static inline void nd_rtrim(string &s)
+{
+    s.erase(find_if(s.rbegin(), s.rend(), [](unsigned char ch) {
+        return !isspace(ch);
+    }).base(), s.end());
+}
+
 int nd_sha1_file(const string &filename, uint8_t *digest)
 {
     sha1 ctx;
@@ -345,6 +372,58 @@ void nd_sha1_to_string(const uint8_t *digest_bin, string &digest_str)
     digest_str.assign(_digest);
 }
 
+bool nd_string_to_mac(const string &src, uint8_t *mac)
+{
+    if (src.size() != ND_STR_ETHALEN) return false;
+
+    uint8_t *p = mac;
+    const char *s = src.c_str();
+
+    for (int i = 0; i < ND_STR_ETHALEN; i += 3, p++) {
+        if (sscanf(s + i, "%2hhx", p) != 1) return false;
+    }
+
+    return true;
+}
+
+sa_family_t nd_string_to_ip(const string &src, sockaddr_storage *ip)
+{
+    sa_family_t family = AF_UNSPEC;
+    struct sockaddr_in *ipv4 = reinterpret_cast<struct sockaddr_in *>(ip);
+    struct sockaddr_in6 *ipv6 = reinterpret_cast<struct sockaddr_in6 *>(ip);
+
+    if (inet_pton(AF_INET, src.c_str(), &ipv4->sin_addr) == 1)
+        family = AF_INET;
+    else if (inet_pton(AF_INET6, src.c_str(), &ipv6->sin6_addr) == 1)
+        family = AF_INET6;
+
+    return family;
+}
+
+bool nd_ip_to_string(const sockaddr_storage &ip, string &dst)
+{
+    char addr[INET6_ADDRSTRLEN];
+    const struct sockaddr_in *ipv4 = reinterpret_cast<const struct sockaddr_in *>(&ip);
+    const struct sockaddr_in6 *ipv6 = reinterpret_cast<const struct sockaddr_in6 *>(&ip);
+
+    switch (ip.ss_family) {
+    case AF_INET:
+        inet_ntop(AF_INET, &ipv4->sin_addr.s_addr, addr, INET_ADDRSTRLEN);
+        break;
+
+    case AF_INET6:
+        inet_ntop(AF_INET6, &ipv6->sin6_addr.s6_addr, addr, INET6_ADDRSTRLEN);
+        break;
+
+    default:
+        return false;
+    }
+
+    dst.assign(addr);
+
+    return true;
+}
+
 void nd_iface_name(const string &iface, string &result)
 {
     result = iface;
@@ -353,6 +432,14 @@ void nd_iface_name(const string &iface, string &result)
         result = iface.substr(0, p);
 }
 
+void nd_capture_filename(const string &iface, string &result)
+{
+    result.clear();
+    size_t p = string::npos;
+    if ((p = iface.find_first_of(",")) != string::npos)
+        result = iface.substr(p + 1);
+}
+
 bool nd_is_ipaddr(const char *ip)
 {
     struct in_addr addr4;
@@ -380,35 +467,83 @@ void nd_private_ipaddr(uint8_t index, struct sockaddr_storage &addr)
 
     switch (rc) {
     case -1:
-        nd_debug_printf("Invalid private address family.\n");
+        nd_dprintf("Invalid private address family.\n");
         break;
     case 0:
-        nd_debug_printf("Invalid private address: %s\n", os.str().c_str());
+        nd_dprintf("Invalid private address: %s\n", os.str().c_str());
         break;
     }
 }
 
 bool nd_load_uuid(string &uuid, const char *path, size_t length)
 {
+    struct stat sb;
     char _uuid[length + 1];
-    FILE *fh = fopen(path, "r");
 
-    if (fh == NULL) {
-        if (ND_DEBUG || errno != ENOENT)
-            nd_printf("Error loading uuid: %s: %s\n", path, strerror(errno));
+    if (stat(path, &sb) == -1) {
+        if (errno != ENOENT) {
+            nd_printf("Error loading uuid: %s: %s\n",
+                path, strerror(errno));
+        }
         return false;
     }
 
-    if (fread((void *)_uuid, 1, length, fh) != length) {
+    if (! S_ISREG(sb.st_mode)) {
+        nd_printf("Error loading uuid: %s: %s\n",
+            path, "Not a regular file");
+        return false;
+    }
+
+    if (sb.st_mode & S_IXUSR) {
+        FILE *ph = popen(path, "r");
+
+        if (ph == NULL) {
+            if (ND_DEBUG || errno != ENOENT) {
+                nd_printf("Error loading uuid from pipe: %s: %s\n",
+                    path, strerror(errno));
+            }
+            return false;
+        }
+
+        size_t bytes = 0;
+
+        bytes = fread((void *)_uuid, 1, length, ph);
+
+        int rc = pclose(ph);
+
+        if (bytes <= 0 || rc != 0) {
+            nd_printf("Error loading uuid from pipe: %s: %s: %d\n",
+                path, "Invalid pipe read", rc);
+            return false;
+        }
+    }
+    else {
+        FILE *fh = fopen(path, "r");
+
+        if (fh == NULL) {
+            if (ND_DEBUG || errno != ENOENT) {
+                nd_printf("Error loading uuid from file: %s: %s\n",
+                    path, strerror(errno));
+            }
+            return false;
+        }
+
+        if (fread((void *)_uuid, 1, length, fh) != length) {
+            fclose(fh);
+            nd_printf("Error reading uuid from file: %s: %s\n",
+                path, strerror(errno));
+            return false;
+        }
+
         fclose(fh);
-        nd_printf("Error reading uuid: %s: %s\n", path, strerror(errno));
-        return false;
+
+        _uuid[length] = '\0';
     }
 
-    fclose(fh);
-    _uuid[length] = '\0';
     uuid.assign(_uuid);
 
+    nd_rtrim(uuid);
+
     return true;
 }
 
@@ -474,15 +609,10 @@ bool nd_save_sink_url(const string &url)
     return true;
 }
 
-void nd_generate_uuid(string &uuid)
+void nd_seed_rng(void)
 {
-    int digit = 0;
-    deque<char> result;
-    uint64_t input = 623714775;
-    unsigned int seed = (unsigned int)time(NULL);
-    const char *clist = { "0123456789abcdefghijklmnpqrstuvwxyz" };
     FILE *fh = fopen("/dev/urandom", "r");
-    ostringstream os;
+    unsigned int seed = (unsigned int)time(NULL);
 
     if (fh == NULL)
         nd_printf("Error opening random device: %s\n", strerror(errno));
@@ -493,6 +623,16 @@ void nd_generate_uuid(string &uuid)
     }
 
     srand(seed);
+}
+
+void nd_generate_uuid(string &uuid)
+{
+    int digit = 0;
+    deque<char> result;
+    uint64_t input = 623714775;
+    const char *clist = { "0123456789abcdefghijklmnpqrstuvwxyz" };
+    ostringstream os;
+
     input = (uint64_t)rand();
     input += (uint64_t)rand() << 32;
 
@@ -580,7 +720,7 @@ int nd_file_load(const string &filename, string &data)
         if (errno != ENOENT)
             throw runtime_error(strerror(errno));
         else {
-            nd_debug_printf("Unable to load file: %s: %s\n",
+            nd_dprintf("Unable to load file: %s: %s\n",
                 filename.c_str(), strerror(errno));
             return -1;
         }
